// Import necessary D3.js modules
import { csv } from 'd3';
import { select, pie, arc, format, scaleOrdinal } from 'd3';
import { schemeCategory10 } from 'd3-scale-chromatic';

// Define the URL of the dataset
const csvUrl =
  'https://raw.githubusercontent.com/kowshiksamudrala08/datasets/main/data_science_salaries.csv';

// Fetch the data from the dataset
const fetchData = async () => {
  const data = await csv(csvUrl);
  return data;
};

// Generate an array of 50 different colors
const sectionColors = schemeCategory10
  .concat(schemeCategory10)
  .concat(schemeCategory10)
  .concat(schemeCategory10)
  .concat(schemeCategory10); // Concatenate the colors arrays to get 50 different colors

// Function to render the pie chart
const renderPieChart = (data) => {
  // Group the data by relevant factors (e.g., job title, level of experience, geographical location) and calculate aggregate salary
  const aggregatedData = data.reduce((acc, curr) => {
    // Use relevant columns for grouping (e.g., job title, level of experience, geographical location)
    const key = curr.job_title; // Change this to the relevant column
    acc[key] = (acc[key] || 0) + parseFloat(curr.salary); // Change this to the relevant column
    return acc;
  }, {});

  // Convert aggregated data to an array of objects
  const pieData = Object.entries(aggregatedData).map(
    ([key, value]) => ({ key, value }),
  );

  // Define SVG dimensions
  const svgWidth = 800;
  const svgHeight = 1000;
  const margin = 5;

  // Create SVG element
  const svg = select('body')
    .append('svg')
    .attr('width', svgWidth)
    .attr('height', svgHeight)
    .append('g') // Append a group element
    .attr(
      'transform',
      `translate(${svgWidth / 2},${svgHeight / 2})`,
    ); // Translate the group to the center

  // Define pie chart parameters
  const radius = Math.min(svgWidth, svgHeight) / 2 - margin;
  const pieGenerator = pie().value((d) => d.value);
  const arcGenerator = arc()
    .innerRadius(0)
    .outerRadius(radius);

  // Create pie slices
  const pieSlices = svg
    .selectAll('path')
    .data(pieGenerator(pieData))
    .enter()
    .append('path')
    .attr('d', arcGenerator)
    .attr(
      'fill',
      (d, i) => sectionColors[i % sectionColors.length],
    ) // Apply colors to slices
    .on('mouseover', function (event, d) {
      // Show tooltip on hover
      const total = d3.sum(pieData.map((d) => d.value));
      const percentage =
        ((d.value / total) * 100).toFixed(2) + '%';
      select(this)
        .append('title')
        .text(
          `${d.data.key}: ${format('.2s')(d.data.value)}, ${percentage}`,
        );
    })
    .on('mouseout', function () {
      // Remove tooltip when not hovered
      select(this).select('title').remove();
    });

  // Add legend
  const legend = svg
    .selectAll('.legend')
    .data(pieData)
    .enter()
    .append('g')
    .attr('class', 'legend')
    .attr(
      'transform',
      (d, i) =>
        `translate(${radius + margin},${(i - pieData.length / 2) * 20})`,
    ); // Position the legend relative to the center

  legend
    .append('rect')
    .attr('x', -18)
    .attr('width', 18)
    .attr('height', 18)
    .attr(
      'fill',
      (d, i) => sectionColors[i % sectionColors.length],
    );

  legend
    .append('text')
    .attr('x', -24)
    .attr('y', 9)
    .attr('dy', '.35em')
    .style('text-anchor', 'end')
    .style('font-size', '12px') // Adjust font size
    .style('font-weight', 'bold') // Add bold font weight
    .text((d) => d.key); // Display the factor (e.g., job title) in the legend
};

// Main function to fetch data and render pie chart
const main = async () => {
  const data = await fetchData();
  renderPieChart(data);
};

// Call the main function
main();
